; Advanced screen routines
; By: Wade Peterson
; 10/11/2000
;
;  This programs gives BASIC programmers
;  easy access to advanaced screen routines
;  that are not possible in BASIC  
;
;///////////////////////////////////////
;// --------------------------------- //
;// | X | Routine                   | //
;// --------------------------------- //
;// | 0 | Clear graph buffer          //
;// | 1 | Copy graph buffer to screen //
;// | 2 | Copy screen to graphbuffer  //
;// | 3 | Copy screen to safe ram     //
;// | 4 | Copy safe ram to screen     //
;// | 5 | Turn screen black           //
;// | 6 | Invert screen               //
;///////////////////////////////////////
;
; *****************************************************
; * To use this, in the BASIC program, store 0-6 to X *
; * Then put Send(9prgmZSCREEN                        *
; *****************************************************
;
; 0 - Clear graph buffer - This simply clears the graph buffer.
;	    Usually, the graph buffer is written to, then copied to the screen.
;       The calcualtor may overwrite the graph buffer, so this routine may not be too reliable.
;
; 1 - Copy graph buffer to screen - This copies the graph buffer to the screen.
;
; 2 - Copy screen to graph buffer - this copies what's on the screen to the buffer. 
;       You could use this to save the screen before you write stuff to it
;       and then use the aboive routine to copy the buffer back to the screen. 
; 
; 3 - This copies the contents of the screen to a block of free RAM. The free RAM is used to save the screen for APD.
;       That is the only time it is accessed, so saving the screen here can be more reliable than
;       saving it to the graph buffer. You could also save one thing to the buffer
;       and save another screen here. 
;
; 4 - This copies the contents of the same RAM to the screen. Example:
;       In BASIC programs, you may save the screen by StorePic 0 or some other pic. 
;       Then, later when you wanted the screen back you would do ClrDraw and RecallPic 0 
;       Well, this is much better. First, it doesn't take up any memory at all (like storing a picture does)
;       Also, when recalling, it writes over the existing screen, so there is no need to clear the screen. 
;       This way the screen doesn't flash, because you never clear it. Here's another example:
;       Say you have some stuff on the screen. Now say you want a menu to pop up over the stuff
;       and when you select something in the menu, it goes away, and the screen goes back to how it was
;       You could store a picture, then have the menu pop up, then clear the sreen and recall the pic
;       However, the screen flashes because you cleared it. Using this routine instead, it doesn't flash
;
; 5 - Turn screen black - this one is easy...it just truns the screen black
;
; 6 - Invert screen - this iverts the color of the screen, i.e. turns white to black and black to white.
;        Very useful.
;

.NOLIST
#define equ .equ
#define EQU .equ
#define end .end
#define lcddataport 11h
#define lcdinstport 10h
#include "ti83asm.inc"
#include "tokens.inc"
.LIST 
.org 9327h
	call _rclX
	call _convop1
	ld a,e	;a=x
	or a
	jp z,_grbufclr
	dec a
	jp z,_grbufcpy_v
	dec a
	jr z,copyscreen1
	dec a
	jr z,copyscreen2
	dec a
	jr z,copytoscreen
	dec a
	ld bc,768
	ld hl,plotsscreen
	ld d,255
	jr z,turnblack
invert:
	ld a,(hl)
	xor d
	ld (hl),a
	inc hl
	dec bc
	ld a,b
	or c
	jr nz,invert
	jp _grbufcpy_v
turnblack:
	ld a,255
	ld (hl),a
	ld de,plotsscreen+1
	ldir
	jp _grbufcpy_v
copyscreen1:
	ld hl,plotsscreen
	jp getscreen
copyscreen2:
	ld hl,savesscreen
	jp getscreen
copytoscreen:
	ld hl,savesscreen
	ld de,plotsscreen
	ld bc,768
	ldir
	jp _grbufcpy_v


;--------= screen grab routine =--------
; Author:       Ian Graf
;               (ian_graf@geocities.com)
; Version:      1.0
; Date:         8/31/98
;
;---------------------------------------
; this routine reads directly from the
; lcd and stores it at hl. there must be
; 768 bytes free at hl. to only grab 756
; bytes (the size of a pic var) change
; the line "cp      0C0h" to
; "cp     0BFh".
;
; example usage:
;
;       ld      hl,plotsscreen
;       call    getscreen
;
; this saves the screen to the graph
; buffer.

;-------------= get screen =------------
; input:        hl - location to save
;               screen
; output:       768 bytes written to
;               location hl
; destroys:     af, bc, d, hl

getscreen:
        di                              ; disable interrupts
        ld      a,07h                   ; set y inc mode
        call    lcdbusy                 ;
        out     (lcdinstport),a         ;
        ld      a,80h                   ; top row
        ld      d,a                     ;
rloop1: call    lcdbusy                 ; set row
        out     (lcdinstport),a         ;
        ld      a,20h                   ; left col
        call    lcdbusy                 ; set col
        out     (lcdinstport),a         ;
        call    lcdbusy                 ; dummy read
        in      a,(lcddataport)         ;
        ld      bc,12*256+lcddataport   ; 12 cols/port 11h
rloop2: call    lcdbusy                 ;
        ini                             ; read col
        jr      nz,rloop2               ;
        inc     d                       ;
        ld      a,d                     ;
        cp      0C0h                    ; bottom row
        jr      nz,rloop1               ;
        ld      a,05h                   ; x inc mode
        call    lcdbusy                 ;
        out     (lcdinstport),a         ;
        ei                              ; enable interrupts
        ret                             ;
;---------------= lcdbusy =-------------

lcdbusy:
        push    af                      ; create a small delay before
        inc     hl                      ; communicating with LCD
        dec     hl                      ;
        pop     af                      ;
        ret                             ;
.end
END